"use strict";

// const { buffer } = require("./util")

var serviceId = "0000FFE0-0000-1000-8000-00805F9B34FB";
var receiveId = "0000FFE1-0000-1000-8000-00805F9B34FB";

function hex2buf(data) {
	var typedArray = new Uint8Array(data.match(/[\da-f]{2}/gi).map(function (h) {
		return parseInt(h, 16);
	}));
	return typedArray.buffer;
}

function buf2hex(buffer) {
	// buffer is an ArrayBuffer
	return Array.prototype.map.call(new Uint8Array(buffer), function (x) {
		return ('00' + x.toString(16)).slice(-2);
	}).join('');
}

/**
 * 
 * @param {设备蓝牙地址，字符串类型} deviceId 
 * @param {发送数据方法，参数为十六进制字符串} sendFunc 
 */
function RfidUtil(deviceId, sendFunc) {
	///构造方法，传入
	var buffer = new Buffer();

	///接收数据调用的方法
	this.onReceiveData = function (res) {
		///接收数据，存放至缓冲区
		var hex = buf2hex(res.value);
		console.log('RX:', hex);
		buffer.write(res.value);
	};

	///接收数据调用的方法
	/**
  * 
  * @param {十六进制字符串} hex 
  */
	this.onReceivehexData = function (hex) {
		///接收数据，存放至缓冲区
		console.log('RX:', hex);
		buffer.write(hex2buf(hex));
	};

	async function delay(milSec) {
		return new Promise(function (resolve) {
			setTimeout(resolve, milSec);
		});
	}

	///发送数据
	// async function send(cmd) {
	// 	this.buffer.clear()
	// 	var value = hex2buf(cmd)
	// 	console.log('TX:', cmd)
	// 	return await wx.writeBLECharacteristicValue({
	// 		deviceId: this.deviceId,
	// 		serviceId: serviceId,
	// 		characteristicId: receiveId,
	// 		value: value
	// 	})
	// }

	///接收返回帧 timeout 等待时间 毫秒
	async function receive(timeout, func) {
		var startTime = new Date(); //结束时间
		while (true) {
			if (buffer.size <= 0) {
				await delay(200);
				continue;
			}
			var start = buffer.indexOf(-86);

			console.log("flag:", start, "pos:", buffer.startPos, "total:", buffer.size);

			if (start >= 0 && buffer.size - start > 9) {
				var value = buffer.typedArray.subarray(start + 6, start + 9);
				var len = (value[2] & 0xFF) << 16 | (value[1] & 0xFF) << 8 | value[0] & 0xFF;

				// console.log('报文数据区长度：', len)

				var end = start + len + 11;
				if (buffer.size >= end) {
					var frame = buffer.typedArray.subarray(start, end);
					var cs = frame[frame.length - 2];
					var cs_check = 0;
					for (var i = 0; i < frame.length - 2; i++) {
						cs_check += frame[i] & 0xff;
					}
					cs_check &= 0xff;

					// console.log('cs: ', cs)
					// console.log('cs_check: ', cs_check)
					// console.log('cs_check: ', (246 - cs_check))

					buffer.startPos = end;
					if (cs == cs_check || cs == cs_check - 256) {
						console.log('接收到报文数据');
						var frame = new Frame(frame);
						///比较功能码
						if (frame.func[0] == func[0] && frame.func[1] == func[1]) {
							return frame;
						}
					}
				} else {
					if (len > 200) {
						buffer.startPos = start + 1;
					}
				}
			}

			var diff = new Date() - startTime.getTime(); //时间差的毫秒数 
			// console.log('时间差：', diff)
			if (diff > timeout) {
				return null;
			}

			await delay(200);
		}
	}

	stop = true;

	//盘点标签
	this.searchTag = async function (callBack) {
		console.log('搜索标签');
		//盘点时间无限
		sendFunc('AA42010001000900000180000101000000007AA5');
		//盘点时间1.5s
		// sendFunc('AA42010001000900000180000101DC0500005BA5')

		if (!stop) {
			return;
		}
		stop = false;
		while (!stop) {
			var frame = await receive(200, [1, -128]);
			if (stop) {
				console.log('结束盘点——————————————————————————');
				break;
			}
			if (frame == null) {
				continue;
			}
			console.log('接收到数据——————————————————', frame);
			var epc = frame.getEpc();
			if (epc != null) {
				callBack(epc);
			}
			if (frame.flag == 1) {
				sendFunc('AA42010001000900000180000101000000007AA5');
			}
		}
	};

	//停止盘点标签
	this.stopSearch = async function () {
		return new Promise(function (resolve) {
			console.log('停止搜索标签');
			stop = true;
			sendFunc('AA4201000100020000048074A5');
			delay(500);
			sendFunc('AA4201000100020000048074A5');
			// delay(500)
			// sendFunc('AA4201000100020000048074A5')
			// delay(500)
			// sendFunc('AA4201000100020000048074A5')
			var frame = receive(5000, [4, -128]);
			console.log('stopSearch: ', frame);
			resolve(true);
		});
	};
}

function Buffer() {
	this.typedArray = new Int8Array(new ArrayBuffer(10240));
	///缓冲区数据长度
	this.size = 0;
	///待解析数据起始位置
	this.startPos = 0;

	this.write = function (data) {
		var temp = new Int8Array(data);
		if (this.size + temp.byteLength >= 10240) {
			this.clear();
		}
		this.typedArray.set(temp, this.size);
		this.size += temp.byteLength;
	};

	this.indexOf = function (value) {
		return this.typedArray.indexOf(value, this.startPos);
	};

	this.read = function (count) {
		this.typedArray = this.typedArray.subarray(count);
		this.size -= count;
	};

	///带解析数据长度
	this.available = function () {
		return this.size - this.startPos;
	};

	this.clear = function () {
		this.size = 0;
		this.startPos = 0;
		this.typedArray.fill(0);
	};
}

///帧格式
function Frame(array) {
	this.property = array[1];
	this.value = array.subarray(6, 9);
	this.len = (this.value[2] & 0xFF) << 16 | (this.value[1] & 0xFF) << 8 | this.value[0] & 0xFF;
	///功能码
	this.func = array.subarray(9, 11);
	///数据区，不包含功能码
	this.data = array.subarray(11, 9 + this.len);

	///是否盘点结束
	this.flag = 0;

	///解析帧格式获取EPC
	this.getEpc = function () {
		var data = this.data;
		if (this.func[0] != 1 && this.func[1] != -128) {
			return null;
		}
		if (data[0] != 0 && data[1] != 0) {
			return null;
		}
		this.flag = data[2];
		if (this.flag == 1) {
			return null;
		}
		var epcLen = data[3];
		var temp = data.subarray(4, 4 + epcLen * 2).reverse();
		var epc = buf2hex(temp);
		return epc;
	};
}
